package com.example.doubledeckerbus;

import javafx.scene.canvas.GraphicsContext;

import java.util.Random;

public abstract class AbstractMap {
    private IDrawingObject _drawingObject = null;
    protected int[][] _map = null;
    protected int _width;
    protected int _height;
    protected float _size_x;
    protected float _size_y;
    protected final Random _random = new Random();
    protected final int _freeRoad = 0;
    protected final int _barrier = 1;

    protected GraphicsContext gc;

    public void CreateMap(int width, int height, IDrawingObject drawingObject, GraphicsContext gc)
    {
        _width = width;
        _height = height;
        _drawingObject = drawingObject;
        this.gc = gc;
        GenerateMap();
        while (!SetObjectOnMap())
        {
            GenerateMap();
        }
        DrawMapWithObject();
    }

    private boolean SetObjectOnMap()
    {
        if (_drawingObject == null || _map == null)
        {
            return false;
        }
        int x = _random.nextInt(0, 10);
        int y = _random.nextInt(0, 10);
        _drawingObject.SetObject(x, y, _width, _height);
        return !CheckCollision();
    }

    private void DrawMapWithObject()
    {
        if (_drawingObject == null || _map == null)
        {
            return;
        }

        for (int i = 0; i < _map.length; ++i)
        {
            for (int j = 0; j < _map[0].length; ++j)
            {
                if (_map[i][j] == _freeRoad)
                {
                    DrawRoadPart(i, j);
                }
                    else if (_map[i][j] == _barrier)
                {
                    DrawBarrierPart(i, j);
                }
            }
        }
        _drawingObject.DrawingObject(gc);
    }

    public void MoveObject(Direction direction)
    {
        _drawingObject.MoveObject(direction);
        if (CheckCollision()) {
            switch (direction) {
                case Left -> _drawingObject.MoveObject(Direction.Right);
                case Right -> _drawingObject.MoveObject(Direction.Left);
                case Up -> _drawingObject.MoveObject(Direction.Down);
                case Down -> _drawingObject.MoveObject(Direction.Up);
            }

        }
        DrawMapWithObject();
    }

    private boolean CheckCollision() {
        var pos = _drawingObject.GetCurrentPosition();
        int startX = (int)((pos.Left) / _size_x);
        int endX = (int)((pos.Right) / _size_x);
        int startY = (int)((pos.Top) / _size_y);
        int endY = (int)((pos.Bottom) / _size_y);

        if (startX < 0 || startY < 0 || endX > _map[0].length || endY > _map.length) {
            return false;
        }

        for (int y = startY; y <= endY + 1; y++)
        {
            for (int x = startX; x <= endX + 1; x++)
            {
                try {
                    if (_map[x][y] == _barrier)
                    {
                        return true;
                    }
                }
                catch (ArrayIndexOutOfBoundsException e) {
                    continue;
                }
            }
        }

        return false;
    }


    protected abstract void GenerateMap();
    protected abstract void DrawRoadPart(int i, int j);
    protected abstract void DrawBarrierPart(int i, int j);
}
