package com.example.doubledeckerbus;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.control.Spinner;
import javafx.scene.input.*;
import javafx.scene.layout.Background;
import javafx.scene.layout.Region;
import javafx.scene.paint.Color;
import javafx.scene.paint.Paint;
import javafx.stage.Stage;

import java.util.function.Consumer;

public class ControllerBusConfig {
    DataFormat colorFormat = DataFormat.lookupMimeType("SerializableColor");
    DataFormat additionalElementFormat = DataFormat.lookupMimeType("IDrawningAdditionalElement");
    private Stage _stage;
    private DrawingBus _bus = null;

    private Event<DrawingBus> addBus;

    public void SetStage(Stage stage)
    {
        _stage = stage;
    }

    public void AddEvent(Consumer<DrawingBus> ev)
    {
        addBus.AddListener(ev);
    }

    private void DrawBus()
    {
        GraphicsContext gc = canvasObject.getGraphicsContext2D();
        gc.clearRect(0, 0, canvasObject.getWidth(), canvasObject.getHeight());
        if (_bus == null)
        {
            return;
        }
        _bus.SetPosition(5, 5, (int)canvasObject.getWidth(), (int)canvasObject.getHeight());
        _bus.DrawTransport(gc);
    }

    @FXML
    private Button buttonCancel;

    @FXML
    private Canvas canvasObject;

    @FXML
    private CheckBox checkBoxSecondStage;

    @FXML
    private CheckBox checkBoxLadder;

    @FXML
    private Spinner<Integer> spinnerSpeed = new Spinner<>();

    @FXML
    private Spinner<Integer> spinnerDoors = new Spinner<>();

    @FXML
    private Spinner<Integer> spinnerWeight = new Spinner<>();

    @FXML
    public void initialize()
    {
        if (colorFormat == null)
        {
            colorFormat = new DataFormat("SerializableColor");
        }
        if (additionalElementFormat == null)
        {
            additionalElementFormat = new DataFormat("IDrawningAdditionalElement");
        }
        addBus = new Event<>();
    }

    @FXML
    void ButtonOk_Click(ActionEvent event) {
        if (_bus != null)
        {
            addBus.Broadcast(_bus);
        }
        if (_stage != null)
        {
            _stage.close();
        }
    }

    @FXML
    void CanvasObject_OnDragDropped(DragEvent event) {
        Dragboard db = event.getDragboard();
        switch (db.getString())
        {
            case "labelSimpleObject" -> _bus = new DrawingBus(spinnerSpeed.getValue(),
                    spinnerWeight.getValue(), Color.WHITE, spinnerDoors.getValue());
            case "labelModifiedObject" -> _bus = new DrawingDDB(spinnerSpeed.getValue(),
                    spinnerWeight.getValue(), Color.WHITE, spinnerDoors.getValue(), Color.BLACK,
                    checkBoxSecondStage.isSelected(), checkBoxLadder.isSelected());
        }
        event.consume();
        DrawBus();
    }

    @FXML
    void CanvasObject_OnDragOver(DragEvent event) {
        if (event.getDragboard().hasString())
        {
            event.acceptTransferModes(TransferMode.COPY_OR_MOVE);
        }
        event.consume();
    }

    @FXML
    void LabelBaseColor_OnDragDropped(DragEvent event) {
        if (_bus == null)
        {
            return;
        }

        Dragboard db = event.getDragboard();

        _bus.Bus.BodyColor = ((SerializableColor)(db.getContent(colorFormat))).getFXColor();
        event.consume();
        DrawBus();
    }

    @FXML
    void LabelColor_OnDragOver(DragEvent event) {
        if (event.getDragboard().hasContent(colorFormat))
        {
            event.acceptTransferModes(TransferMode.COPY_OR_MOVE);
        }
        event.consume();
    }

    @FXML
    void LabelDopColor_OnDragDropped(DragEvent event) {
        if (_bus == null || !(_bus.Bus instanceof EntityDDB ddb))
        {
            return;
        }

        Dragboard db = event.getDragboard();

        ddb.ExtraColor = ((SerializableColor)(db.getContent(colorFormat))).getFXColor();
        event.consume();
        DrawBus();
    }

    @FXML
    void LabelObject_OnDragDetected(MouseEvent event) {
        Label labelObject = (Label)(event.getSource());
        Dragboard db = labelObject.startDragAndDrop(TransferMode.ANY);

        ClipboardContent content = new ClipboardContent();
        content.putString(((Label)(event.getSource())).getId());
        db.setContent(content);

        event.consume();
    }

    @FXML
    void RegionColor_OnDragDetected(MouseEvent event) {
        Region region = (Region)(event.getSource());
        Dragboard db = region.startDragAndDrop(TransferMode.ANY);

        Background regionBackground = region.getBackground();
        Paint regionPaint = regionBackground.getFills().get(0).getFill();
        if (regionPaint instanceof Color)
        {
            ClipboardContent content = new ClipboardContent();
            content.put(colorFormat, new SerializableColor((Color)regionPaint));
            db.setContent(content);
        }
        event.consume();
    }

}
