package com.example.doubledeckerbus;

import javafx.beans.InvalidationListener;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.*;
import javafx.scene.layout.AnchorPane;
import javafx.scene.paint.Color;

import java.io.IOException;
import java.util.Optional;
import java.util.Random;

import static com.example.doubledeckerbus.ControllerMapWithSetBus._mapBusesCollectionGeneric;

public class ControllerPolymorph {
    protected DrawingBus _bus;
    protected DrawingPolymorphBus<EntityBus, IDrawingDoors> polymorphBus;
    public DrawingBus SelectedBus;
    protected ObservableList<Integer> countOfDoors = FXCollections.observableArrayList(3, 4, 5);

    @FXML
    protected Button buttonCreate;

    @FXML
    protected Canvas canvasBus;

    @FXML
    protected AnchorPane pictureBoxBus;

    @FXML
    protected Label statusColor;

    @FXML
    protected Label statusSpeed;

    @FXML
    protected Label statusWeight;

    @FXML
    protected ChoiceBox<Integer> choiceDoors;

    @FXML
    void ButtonCreate_Click(ActionEvent event) {
        pictureBoxBus.widthProperty().addListener(listener);
        pictureBoxBus.heightProperty().addListener(listener);

        Random rnd = new Random();
        IDrawingDoors door = null;
        switch (rnd.nextInt(3)) {
            case (0) -> door = new DrawingDoors();
            case (1) -> door = new DrawingEllipsoidDoors();
            case (2) -> door = new DrawingTriangleDoors();
        }
        door.setCountOfDoors(choiceDoors.getValue());

        EntityBus bus = new EntityBus(rnd.nextInt(300), rnd.nextInt(2000),
                Color.rgb(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)));

        polymorphBus.AddDoors(door);
        polymorphBus.AddEntity(bus);

        _bus = polymorphBus.CreateBus();
        SetData();
        BorderChanged();
    }

    void SetData() {
        Random rnd = new Random();
        _bus.SetPosition(rnd.nextInt(10, 100), rnd.nextInt(10, 100), (int) pictureBoxBus.getWidth(), (int) pictureBoxBus.getHeight());
        statusSpeed.setText("Скорость: %s".formatted(_bus.Bus.Speed));
        statusWeight.setText("Вес: %s".formatted(_bus.Bus.Weight));
        statusColor.setText("Цвет: %s".formatted(_bus.Bus.BodyColor));
    }

    @FXML
    public void initialize() {
        polymorphBus = new DrawingPolymorphBus<>(100, 100);
        choiceDoors.setItems(countOfDoors);
        choiceDoors.setValue(3);

    }
    InvalidationListener listener = o -> BorderChanged();

    private void Draw()
    {
        GraphicsContext gc = canvasBus.getGraphicsContext2D();
        gc.setFill(Color.WHITE);
        gc.fillRect(0, 0, pictureBoxBus.getWidth(), pictureBoxBus.getHeight());
        _bus.DrawTransport(gc);
    }

    void BorderChanged() {
        canvasBus.setWidth(pictureBoxBus.getWidth());
        canvasBus.setHeight(pictureBoxBus.getHeight());
        _bus.ChangeBorders((int) pictureBoxBus.getWidth(), (int) pictureBoxBus.getHeight());
        Draw();
    }

    @FXML
    private void ButtonCreateExtra_Click(ActionEvent event) {
        pictureBoxBus.widthProperty().addListener(listener);
        pictureBoxBus.heightProperty().addListener(listener);
        Random rnd = new Random();
        IDrawingDoors door = null;
        switch (rnd.nextInt(3)) {
            case (0) -> door = new DrawingDoors();
            case (1) -> door = new DrawingEllipsoidDoors();
            case (2) -> door = new DrawingTriangleDoors();
        }
        door.setCountOfDoors(choiceDoors.getValue());

        EntityDDB ddb_bus = new EntityDDB(rnd.nextInt(300), rnd.nextInt(2000),
                Color.rgb(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)),
                Color.rgb(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)),
                rnd.nextBoolean(), rnd.nextBoolean());

        polymorphBus.AddDoors(door);
        polymorphBus.AddEntity(ddb_bus);

        _bus = polymorphBus.CreateBus();
        SetData();
        BorderChanged();
    }
    @FXML
    private void ButtonSelectBus_Click(ActionEvent event) throws IOException {
        SelectedBus = _bus;

        if (SelectedBus == null) {
            Alert alert = new Alert(Alert.AlertType.INFORMATION);
            alert.setTitle("SelectBus");
            alert.setContentText("Вы не создали объект");
            Optional<ButtonType> option = alert.showAndWait();
        }
        else {

            DrawingObjectBus bus = new DrawingObjectBus(SelectedBus);

            if (_mapBusesCollectionGeneric.add(bus) != -1) {
                Alert alert = new Alert(Alert.AlertType.INFORMATION);
                alert.setTitle("SelectBus");
                alert.setContentText("Вы создали объект");
                Optional<ButtonType> option = alert.showAndWait();
            } else {
                Alert alert = new Alert(Alert.AlertType.INFORMATION);
                alert.setTitle("SelectBus");
                alert.setContentText("Не удалось добавить объект");
                Optional<ButtonType> option = alert.showAndWait();
            }
        }

        FXMLLoader fxmlLoader = new FXMLLoader(Form.class.getResource("FormMapWithSetBus.fxml"));
        Scene scene = new Scene(fxmlLoader.load());
        Form.myStage.setTitle("DoubleDeckerBus");
        Form.myStage.setScene(scene);
        Form.myStage.show();

    }
}
