package com.example.doubledeckerbus;

import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;

import java.util.Random;

public class DrawingBus {
    public EntityBus Bus;

    public IDrawingDoors Doors;
    int _speed;
    float _weight;
    Color _bodyColor;
    int _countOfDoors;

    private static final int _null = -1000;
    protected float _startPosX;
    protected float _startPosY;
    private int _pictureWidth;
    private int _pictureHeight;
    private static final int _busWidth = 100;
    private static final int _busHeight = 50;

    public DrawingBus(int speed, float weight, Color bodyColor, int countOfDoors) {
        Bus = CreateBus(speed, weight, bodyColor, countOfDoors);
    }

    private EntityBus CreateBus(int speed, float weight, Color bodyColor, int countOfDoors) {
        _speed = speed;
        _weight = weight;
        _bodyColor = bodyColor;
        _countOfDoors = countOfDoors;
        EntityBus bus = new EntityBus(speed, weight, bodyColor);
        switch (new Random().nextInt(3)) {
            case 0 -> Doors = new DrawingTriangleDoors();
            case 1 -> Doors = new DrawingDoors();
            case 2 -> Doors = new DrawingEllipsoidDoors();
        }
        Doors.setCountOfDoors(countOfDoors);
        return bus;
    }


    public DrawingBus(int speed, float weight, Color bodyColor) {
        Bus = new EntityBus(speed, weight, bodyColor);
    }

    public DrawingBus(EntityBus bus, IDrawingDoors doors) {
        Bus = bus;
        Doors = doors;
    }

    public void SetPosition(int x, int y, int width, int height) {
        if (width <= _busWidth || height <= _busHeight) {
            _pictureWidth = _null;
            _pictureHeight = _null;
            return;
        }

        _startPosX = x;
        _startPosY = y;
        _pictureWidth = width;
        _pictureHeight = height;
    }

    public void MoveTransport(Direction direction)
    {
        if (_pictureWidth == _null || _pictureHeight == _null)
        {
            return;
        }

        switch (direction)
        {
            case Right:
                if (_startPosX + _busWidth + Bus.Step < _pictureWidth) {
                    _startPosX += Bus.Step;
                }
                break;
            case Left:
                if (_startPosX - Bus.Step >= 0) {
                    _startPosX -= Bus.Step;
                }
                break;
            case Up:
                if (_startPosY - Bus.Step >= 0) {
                    _startPosY -= Bus.Step;
                }
                break;
            case Down:
                if (_startPosY + _busHeight + Bus.Step < _pictureHeight) {
                    _startPosY += Bus.Step;
                }
                break;
            default:
                break;
        }
    }

    public void DrawTransport(GraphicsContext gc)
    {
        if (_startPosX < 0 || _startPosY < 0
                || _pictureHeight == _null  || _pictureWidth == _null)
        {
            return;
        }


        gc.setFill(Bus.BodyColor);
        gc.fillRect(_startPosX, _startPosY + 10, 100, 30);

        //Колеса
        gc.setFill(Color.BLACK);
        gc.fillOval(_startPosX + 7, _startPosY + 35, 10, 10);
        gc.fillOval(_startPosX + 77, _startPosY + 35, 10, 10);

        //окна
        gc.setFill(Color.BLUE);
        gc.fillOval(_startPosX + 10, _startPosY + 15, 10, 15);
        gc.fillOval(_startPosX + 50, _startPosY + 15, 10, 15);
        gc.fillOval(_startPosX + 70, _startPosY + 15, 10, 15);
        gc.fillOval(_startPosX + 90, _startPosY + 15, 10, 15);

        //Дверь
        Doors.DrawDoors(gc, (int)_startPosX, (int)_startPosY);
    }

    public void ChangeBorders(int width, int height) {
        _pictureWidth = width;
        _pictureHeight = height;

        if (_pictureWidth <= _busWidth || _pictureHeight <= _busHeight){
            _pictureWidth = _null;
            _pictureHeight = _null;
            return;
        }

        if (_startPosX + _busWidth > _pictureWidth) {
            _startPosX = _pictureWidth - _busWidth;

        }
        if (_startPosY + _busHeight > _pictureHeight) {
            _startPosY = _pictureHeight - _busHeight;
        }
    }

    public Position GetCurrentPosition() {
        return new Position(_startPosX, _startPosX + _busWidth, _startPosY, _startPosY + _busHeight);
    }
}
