package com.example.doubledeckerbus;

import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;

import java.util.Stack;

public class MapWithSetBusesGeneric<T extends IDrawingObject, U extends AbstractMap> {
    private final int _pictureWidth;
    private final int _pictureHeight;
    private final int _placeSizeWidth = 210;
    private final int _placeSizeHeight = 90;
    private final SetBusesGeneric<T> _setBuses;
    private final Stack<T> _deletedBuses;
    private U _map;

    public MapWithSetBusesGeneric(int picWidth, int picHeight, U map)
    {
        int width = picWidth / _placeSizeWidth;
        int height = picHeight / _placeSizeHeight;
        _setBuses = new SetBusesGeneric<T>(width * height);
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _map = map;
        _deletedBuses = new Stack<>();
    }

    public void changeMap(U map) {
        _map = map;
    }

    public int add(T bus)
    {
        return _setBuses.Insert(bus);
    }

    public T remove(int position)
    {
        T deletedBus = _setBuses.Remove(position);
        _deletedBuses.push(deletedBus);
        return deletedBus;
    }

    public T getDeletedBus(){
        if(_deletedBuses.empty())
            return null;
        return _deletedBuses.pop();
    }

    public void ShowSet(GraphicsContext gc)
    {
        DrawBackground(gc);
        DrawBuses(gc);
    }

    public void ShowOnMap(GraphicsContext gc)
    {
        Shaking();
        for (var bus: _setBuses.GetBuses())
        {
            if (bus != null)
            {
                _map.CreateMap(_pictureWidth, _pictureHeight, bus, gc);
            }
        }

    }

    public void MoveObject(Direction direction)
    {
        if (_map != null)
        {
            _map.MoveObject(direction);
        }
    }

    private void Shaking()
    {
        int j = _setBuses.Count() - 1;
        for (int i = 0; i < _setBuses.Count(); i++)
        {
            if (_setBuses.Get(i) == null)
            {
                for (; j > i; j--)
                {
                    var bus = _setBuses.Get(j);
                    if (bus != null)
                    {
                        _setBuses.Insert(bus, i);
                        _setBuses.Remove(j);
                        break;
                    }
                }
                if (j <= i)
                {
                    return;
                }
            }
        }
    }

    private void DrawBackground(GraphicsContext gc)
    {
        gc.setFill(Color.BLACK);
        for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
        {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
            {//линия рамзетки места
                gc.strokeLine(i * _placeSizeWidth, j * _placeSizeHeight,
                        i * _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);
            }
            gc.strokeLine(i * _placeSizeWidth, 0, i * _placeSizeWidth, (_pictureHeight / _placeSizeHeight) * _placeSizeHeight);
        }
    }

    private void DrawBuses(GraphicsContext gc) {
        int width = _pictureWidth / _placeSizeWidth;
        int height = _pictureHeight / _placeSizeHeight;

        int currentWidth = width - 1;
        int currentHeight = 0;

        for (var bus: _setBuses.GetBuses()) {
            if (bus == null){
                continue;
            }

            bus.SetObject(currentWidth * _placeSizeWidth,
                    currentHeight * _placeSizeHeight,
                    _pictureWidth, _pictureHeight);
            bus.DrawingObject(gc);

            if (currentWidth > 0)
                currentWidth--;
            else {
                currentWidth = width - 1;
                currentHeight++;
            }
            if (currentHeight > height) return;
        }
    }

    public int getCount() {
        return _setBuses.Count();
    }

    public T getBus(int ind){
        return _setBuses.Get(ind);
    }
}

